@@ -2,7 +2,8 @@
  *  acpid.h - ACPI daemon
  *
  *  Copyright (C) 1999-2000 Andrew Henroid
- *  Copyright (C) 2001 Sun Microsystems (thockin@sun.com)
+ *  Copyright (C) 2001 Sun Microsystems
+ *  Portions Copyright (C) 2004 Tim Hockin (thockin@hockin.org)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,7 +32,6 @@
 #define ACPI_PROCDIR 		"/proc/acpi"
 #define ACPI_EVENTFILE		ACPI_PROCDIR "/event"
 #define ACPI_CONFDIR		"/etc/acpi/events"
-#define ACPI_LOGFILE		"/var/log/acpid"
 #define ACPI_SOCKETFILE		"/var/run/acpid.socket"
 #define ACPI_SOCKETMODE		0666
 #define ACPI_MAX_ERRS		5
@@ -42,14 +42,14 @@
  * acpid.c
  */
 extern int acpid_debug;
-extern int acpid_log(const char *fmt, ...);
+extern int acpid_log(int level, const char *fmt, ...);
 
 /*
  * event.c
  */
 extern int acpid_read_conf(const char *confdir);
 extern int acpid_add_client(int client, const char *origin);
-extern int acpid_cleanup_rules(void);
+extern int acpid_cleanup_rules(int do_detach);
 extern int acpid_handle_event(const char *event);
 
 #endif /* ACPID_H__ */
