@@ -3,6 +3,7 @@
  *
  *  Copyright (C) 2000 Andrew Henroid
  *  Copyright (C) 2001 Sun Microsystems (thockin@sun.com)
+ *  Copyright (C) 2004 Tim Hockin (thockin@hockin.org)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +23,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
-#include <libgen.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -80,6 +80,20 @@ static int safe_write(int fd, const char *buf, int len);
 static char *parse_cmd(const char *cmd, const char *event);
 static int check_escapes(const char *str);
 
+/* helper */
+static int
+path_is_dir(const char *path)
+{
+	struct stat s;
+
+	if (stat(path, &s)) {
+		acpid_log(LOG_ERR, "stat(\"%s\"): %s\n",
+		    path, strerror(errno));
+		return 0;
+	}
+	return S_ISDIR(s.st_mode);
+}
+
 /*
  * read in all the configuration files
  */
@@ -90,14 +104,12 @@ acpid_read_conf(const char *confdir)
 	struct dirent *dirent;
 	char *file = NULL;
 	int nrules = 0;
-	char *basen = NULL;
-	regex_t preg;
 
 	lock_rules();
 
 	dir = opendir(confdir);
 	if (!dir) {
-		acpid_log("ERR: opendir(%s): %s\n",
+		acpid_log(LOG_ERR, "opendir(%s): %s\n",
 			confdir, strerror(errno));
 		unlock_rules();
 		return -1;
@@ -107,52 +119,43 @@ acpid_read_conf(const char *confdir)
 	while ((dirent = readdir(dir))) {
 		int len;
 		struct rule *r;
-		struct stat stat_buf;
 
 		if (dirent->d_name[0] == '.')
 			continue; /* skip dotfiles */
 
-		len = strlen(dirent->d_name) + strlen(confdir) + 2;
+		len = strlen(dirent->d_name);
+
+		if (dirent->d_name[len - 1] == '~') {
+			continue; /* skip editor backup files */
+		}
+
+		len += strlen(confdir) + 2;
+
 		file = (char *)malloc(len);
 		if (!file) {
-			acpid_log("ERR: malloc(): %s\n", strerror(errno));
+			acpid_log(LOG_ERR, "malloc(): %s\n", strerror(errno));
 			unlock_rules();
 			return -1;
 		}
 		snprintf(file, len, "%s/%s", confdir, dirent->d_name);
 
-		/* allow only regular files and symlinks to these */
-		if (stat(file, &stat_buf) != 0) {
-			acpid_log("ERR: stat(%s): %s\n", file, strerror(errno));
-			unlock_rules();
-			return -1;
-		}
-		if (!S_ISREG(stat_buf.st_mode)) {
-			acpid_log("DBG: skipping non-file %s\n", file);
+		if (path_is_dir(file)) {
 			free(file);
-			continue;
+			continue; /* skip subdirs */
 		}
 
-		/* check for run-parts style filename */
-		basen = basename(file);
-		if (regcomp(&preg, "^[a-zA-Z0-9_-]+$", RULE_REGEX_FLAGS) == 0){
-			if (regexec(&preg, basen, 0, NULL, 0) == 0){
-				r = parse_file(file);
-				if (r) {
-					enlist_rule(&cmd_list, r);
-					nrules++;
-				}
-			} else {
-				acpid_log("DBG: ignoring conf file %s\n", file);
-			}
-
+		r = parse_file(file);
+		if (r) {
+			enlist_rule(&cmd_list, r);
+			nrules++;
 		}
 		free(file);
 	}
 	closedir(dir);
 	unlock_rules();
 
-	acpid_log("%d rule%s loaded\n", nrules, (nrules == 1)?"":"s");
+	acpid_log(LOG_INFO, "%d rule%s loaded\n",
+	    nrules, (nrules == 1)?"":"s");
 
 	return 0;
 }
@@ -161,7 +164,7 @@ acpid_read_conf(const char *confdir)
  * cleanup all rules
  */
 int
-acpid_cleanup_rules(void)
+acpid_cleanup_rules(int do_detach)
 {
 	struct rule *p;
 	struct rule *next;
@@ -169,17 +172,19 @@ acpid_cleanup_rules(void)
 	lock_rules();
 
 	if (acpid_debug >= 3) {
-		acpid_log("DBG: cleaning up rules\n");
+		acpid_log(LOG_DEBUG, "cleaning up rules\n");
 	}
 
-	/* tell our clients to buzz off */
-	p = client_list.head;
-	while (p) {
-		next = p->next;
-		delist_rule(&client_list, p);
-		close(p->action.fd);
-		free_rule(p);
-		p = next;
+	if (do_detach) {
+		/* tell our clients to buzz off */
+		p = client_list.head;
+		while (p) {
+			next = p->next;
+			delist_rule(&client_list, p);
+			close(p->action.fd);
+			free_rule(p);
+			p = next;
+		}
 	}
 
 	/* clear out our conf rules */
@@ -205,12 +210,12 @@ parse_file(const char *file)
 	struct rule *r;
 
 	if (acpid_debug) {
-		acpid_log("DBG: parsing conf file %s\n", file);
+		acpid_log(LOG_DEBUG, "parsing conf file %s\n", file);
 	}
 
 	fp = fopen(file, "r");
 	if (!fp) {
-		acpid_log("ERR: fopen(%s): %s\n", file, strerror(errno));
+		acpid_log(LOG_ERR, "fopen(%s): %s\n", file, strerror(errno));
 		return NULL;
 	}
 
@@ -223,14 +228,14 @@ parse_file(const char *file)
 	r->type = RULE_CMD;
 	r->origin = strdup(file);
 	if (!r->origin) {
-		acpid_log("ERR: strdup(): %s\n", strerror(errno));
+		acpid_log(LOG_ERR, "strdup(): %s\n", strerror(errno));
 		free_rule(r);
 		fclose(fp);
 		return NULL;
 	}
 
 	/* read each line */
-	while (!feof(fp) && !ferror(fp)) {
+	while (!feof(fp)) {
 		char *p = buf;
 		char key[64];
 		char val[512];
@@ -256,19 +261,20 @@ parse_file(const char *file)
 		/* quick parse */
 		n = sscanf(p, "%63[^=\n]=%255[^\n]", key, val);
 		if (n != 2) {
-			acpid_log("can't parse %s at line %d\n",
-				file, line);
+			acpid_log(LOG_WARNING, "can't parse %s at line %d\n",
+			    file, line);
 			continue;
 		}
 		if (acpid_debug >= 3) {
-			acpid_log("DBG:    key=\"%s\" val=\"%s\"\n", key, val);
+			acpid_log(LOG_DEBUG, "    key=\"%s\" val=\"%s\"\n",
+			    key, val);
 		}
 		/* handle the parsed line */
 		if (!strcasecmp(key, "event")) {
 			int rv;
 			r->event = malloc(sizeof(regex_t));
-			if (!r) {
-				acpid_log("ERR: malloc(): %s\n",
+			if (!r->event) {
+				acpid_log(LOG_ERR, "malloc(): %s\n",
 					strerror(errno));
 				free_rule(r);
 				fclose(fp);
@@ -278,35 +284,38 @@ parse_file(const char *file)
 			if (rv) {
 				char buf[128];
 				regerror(rv, r->event, buf, sizeof(buf));
-				acpid_log("ERR: regcomp(): %s\n", buf);
+				acpid_log(LOG_ERR, "regcomp(): %s\n", buf);
 				free_rule(r);
 				fclose(fp);
 				return NULL;
 			}
 		} else if (!strcasecmp(key, "action")) {
 			if (check_escapes(val) < 0) {
-				acpid_log("ERR: can't load file %s\n", file);
+				acpid_log(LOG_ERR, "can't load file %s\n",
+				    file);
 				free_rule(r);
 				fclose(fp);
 				return NULL;
 			}
 			r->action.cmd = strdup(val);
 			if (!r->action.cmd) {
-				acpid_log("ERR: strdup(): %s\n",
+				acpid_log(LOG_ERR, "strdup(): %s\n",
 					strerror(errno));
 				free_rule(r);
 				fclose(fp);
 				return NULL;
 			}
 		} else {
-			acpid_log("unknown option '%s' in %s at line %d\n",
-				key, file, line);
+			acpid_log(LOG_WARNING,
+			    "unknown option '%s' in %s at line %d\n",
+			    key, file, line);
 			continue;
 		}
 	}
 	if (!r->event || !r->action.cmd) {
 		if (acpid_debug) {
-			acpid_log("DBG: skipping incomplete file %s\n", file);
+			acpid_log(LOG_DEBUG, "skipping incomplete file %s\n",
+			    file);
 		}
 		free_rule(r);
 		return NULL;
@@ -322,7 +331,7 @@ acpid_add_client(int clifd, const char *origin)
 	struct rule *r;
 	int nrules = 0;
 
-	acpid_log("client connected from %s\n", origin);
+	acpid_log(LOG_NOTICE, "client connected from %s\n", origin);
 
 	r = parse_client(clifd);
 	if (r) {
@@ -331,7 +340,8 @@ acpid_add_client(int clifd, const char *origin)
 		nrules++;
 	}
 
-	acpid_log("%d client rule%s loaded\n", nrules, (nrules == 1)?"":"s");
+	acpid_log(LOG_INFO, "%d client rule%s loaded\n",
+	    nrules, (nrules == 1)?"":"s");
 
 	return 0;
 }
@@ -350,8 +360,8 @@ parse_client(int client)
 	r->type = RULE_CLIENT;
 	r->action.fd = client;
 	r->event = malloc(sizeof(regex_t));
-	if (!r) {
-		acpid_log("ERR: malloc(): %s\n", strerror(errno));
+	if (!r->event) {
+		acpid_log(LOG_ERR, "malloc(): %s\n", strerror(errno));
 		free_rule(r);
 		return NULL;
 	}
@@ -359,7 +369,7 @@ parse_client(int client)
 	if (rv) {
 		char buf[128];
 		regerror(rv, r->event, buf, sizeof(buf));
-		acpid_log("ERR: regcomp(): %s\n", buf);
+		acpid_log(LOG_ERR, "regcomp(): %s\n", buf);
 		free_rule(r);
 		return NULL;
 	}
@@ -409,7 +419,7 @@ new_rule(void)
 
 	r = malloc(sizeof(*r));
 	if (!r) {
-		acpid_log("ERR: malloc(): %s\n", strerror(errno));
+		acpid_log(LOG_ERR, "malloc(): %s\n", strerror(errno));
 		return NULL;
 	}
 
@@ -467,8 +477,9 @@ acpid_handle_event(const char *event)
 			if (!regexec(p->event, event, 0, NULL, 0)) {
 				/* a match! */
 				if (acpid_debug) {
-					acpid_log("DBG: rule from %s matched\n",
-						p->origin);
+					acpid_log(LOG_DEBUG,
+					    "rule from %s matched\n",
+					    p->origin);
 				}
 				nrules++;
 				if (p->type == RULE_CMD) {
@@ -476,13 +487,15 @@ acpid_handle_event(const char *event)
 				} else if (p->type == RULE_CLIENT) {
 					do_client_rule(p, event);
 				} else {
-					acpid_log("unknown rule type: %d\n",
-						p->type);
+					acpid_log(LOG_WARNING,
+					    "unknown rule type: %d\n",
+					    p->type);
 				}
 			} else {
 				if (acpid_debug >= 3) {
-					acpid_log("DBG: rule from %s did not "
-						"match\n", p->origin);
+					acpid_log(LOG_DEBUG,
+					    "rule from %s did not match\n",
+					    p->origin);
 				}
 			}
 			p = pnext;
@@ -492,7 +505,7 @@ acpid_handle_event(const char *event)
 	unlock_rules();
 
 	if (acpid_debug) {
-		acpid_log("DBG: %d total rule%s matched\n",
+		acpid_log(LOG_DEBUG, "%d total rule%s matched\n",
 			nrules, (nrules == 1)?"":"s");
 	}
 
@@ -518,7 +531,7 @@ static void
 lock_rules(void)
 {
 	if (acpid_debug >= 4) {
-		acpid_log("DBG: blocking signals for rule lock\n");
+		acpid_log(LOG_DEBUG, "blocking signals for rule lock\n");
 	}
 	sigprocmask(SIG_BLOCK, signals_handled(), NULL);
 }
@@ -527,7 +540,7 @@ static void
 unlock_rules(void)
 {
 	if (acpid_debug >= 4) {
-		acpid_log("DBG: unblocking signals for rule lock\n");
+		acpid_log(LOG_DEBUG, "unblocking signals for rule lock\n");
 	}
 	sigprocmask(SIG_UNBLOCK, signals_handled(), NULL);
 }
@@ -546,12 +559,12 @@ do_cmd_rule(struct rule *rule, const char *event)
 	pid = fork();
 	switch (pid) {
 	case -1:
-		acpid_log("ERR: fork(): %s\n", strerror(errno));
+		acpid_log(LOG_ERR, "fork(): %s\n", strerror(errno));
 		return -1;
 	case 0: /* child */
 		/* parse the commandline, doing any substitutions needed */
 		action = parse_cmd(rule->action.cmd, event);
-		acpid_log("executing action \"%s\"\n", action);
+		acpid_log(LOG_INFO, "executing action \"%s\"\n", action);
 
 		/* reset signals */
 		signal(SIGHUP, SIG_DFL);
@@ -561,23 +574,27 @@ do_cmd_rule(struct rule *rule, const char *event)
 		signal(SIGPIPE, SIG_DFL);
 		sigprocmask(SIG_UNBLOCK, signals_handled(), NULL);
 
-		acpid_log("BEGIN HANDLER MESSAGES\n");
+		if (acpid_debug)
+			fprintf(stdout, "BEGIN HANDLER MESSAGES\n");
 		execl("/bin/sh", "/bin/sh", "-c", action, NULL);
 		/* should not get here */
-		acpid_log("ERR: execl(): %s\n", strerror(errno));
+		acpid_log(LOG_ERR, "execl(): %s\n", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
 	/* parent */
 	waitpid(pid, &status, 0);
-	acpid_log("END HANDLER MESSAGES\n");
+	if (acpid_debug)
+		fprintf(stdout, "END HANDLER MESSAGES\n");
 	if (WIFEXITED(status)) {
-		acpid_log("action exited with status %d\n",
-			WEXITSTATUS(status));
+		acpid_log(LOG_INFO, "action exited with status %d\n",
+		    WEXITSTATUS(status));
 	} else if (WIFSIGNALED(status)) {
-		acpid_log("action exited on signal %d\n", WTERMSIG(status));
+		acpid_log(LOG_INFO, "action exited on signal %d\n",
+		    WTERMSIG(status));
 	} else {
-		acpid_log("action exited with status %d\n", status);
+		acpid_log(LOG_INFO, "action exited with status %d\n",
+		    status);
 	}
 
 	return 0;
@@ -589,13 +606,15 @@ do_client_rule(struct rule *rule, const char *event)
 	int r;
 	int client = rule->action.fd;
 
-	acpid_log("notifying client %s\n", rule->origin);
+	acpid_log(LOG_INFO, "notifying client %s\n", rule->origin);
 
 	r = safe_write(client, event, strlen(event));
 	if (r < 0 && errno == EPIPE) {
 		/* closed */
-		acpid_log("client has disconnected\n");
+		acpid_log(LOG_NOTICE, "client has disconnected\n");
 		delist_rule(&client_list, rule);
+		close(rule->action.fd);
+		free_rule(rule);
 		return -1;
 	}
 	safe_write(client, "\n", 1);
@@ -629,7 +648,7 @@ safe_write(int fd, const char *buf, int len)
 	if (!ntries) {
 		/* crap */
 		if (acpid_debug >= 2) {
-			acpid_log("uh-oh! safe_write() timed out\n");
+			acpid_log(LOG_ERR, "uh-oh! safe_write() timed out\n");
 		}
 		return r;
 	}
@@ -666,7 +685,7 @@ parse_cmd(const char *cmd, const char *event)
 		buf[i++] = *p++;
 	}
 	if (acpid_debug >= 2) {
-		acpid_log("DBG: expanded \"%s\" -> \"%s\"\n", cmd, buf);
+		acpid_log(LOG_DEBUG, "expanded \"%s\" -> \"%s\"\n", cmd, buf);
 	}
 
 	return buf;
@@ -684,10 +703,12 @@ check_escapes(const char *str)
 		if (*p == '%') {
 			p++;
 			if (!*p) {
-				acpid_log("invalid escape at EOL\n");
+				acpid_log(LOG_WARNING,
+				    "invalid escape at EOL\n");
 				return -1;
 			} else if (*p != '%' && *p != 'e') {
-				acpid_log("invalid escape \"%%%c\"\n", *p);
+				acpid_log(LOG_WARNING,
+				    "invalid escape \"%%%c\"\n", *p);
 				r = -1;
 			}
 		}
